require 'fileutils'
require 'pathname'

class Torque

  ##
  # Creates a (limited) interface through which Torque can interact with the local file system
  #
  # Supports:
  #
  # * File creation, reading, line-by-line iteration, and overwriting
  # * Directory creation
  # * Pathname checking
  #
  class FileSystem

    def initialize
      # Do nothing; the file system's properties are by definition global
    end

    ##
    # Creates a file, optionally overwriting an existing file to do so
    def file_create(filename, over=false)
      if !path_exist? filename || over
        File.open(filename, "w")
      else
        raise IOError.new "File #{filename} already exists"
      end
    end

    ##
    # @return An iterator over each line of a file
    def file_each_line(filename)
      File.open(filename, "r").each_line
    end

    ##
    # @return The contents of a file
    def file_read(filename)
      File.read(filename)
    end

    ##
    # Writes a string to file
    def file_write(filename, string)
      File.write(filename, string)
    end

    ##
    # Creates a directory
    def mkdir(dirname)
      FileUtils.mkdir(dirname)
    end

    ##
    # Creates a directory, creating intermediate directories as needed
    def mkdir_p(dirname)
      FileUtils.mkdir_p(dirname)
    end

    ##
    # @param pathname The pathname to test
    #
    # @return True if the pathname exists, else false
    def path_exist?(pathname)
      Pathname.new(pathname).exist?
    end

  end
end
